/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_prog.h"
#include "cone_error.h"

#include <string.h>
#include <stdlib.h>

#define MAX_LINE_LENGTH 4096

struct cone_prog*  
cone_prog_create(const char* path)
{
  FILE* file;
  char  line[MAX_LINE_LENGTH];
  char* name;
  int   i, tmp;
  long  temp;
  struct cone_prog* prog;
  
  file = fopen(path, "r");
  
  if ( file == NULL )
    cone_error();

  prog = malloc(sizeof(struct cone_prog));
  if ( prog == NULL )
    cone_error();

  
  if ( fgets(line, MAX_LINE_LENGTH, file) == NULL )
    cone_error();
    
  
  if ( strcmp(strtok(line, " "), "PROGRAM") != 0 )
    cone_error_msg("invalid program-specification string");
  
  prog->nmods = atol(strtok(NULL, " "));
  if ( prog->nmods > 0 )
    {
      prog->modv  = (struct cone_mod**) calloc(prog->nmods, sizeof(struct cone_mod*));
      if ( prog->modv == NULL )
	cone_error();
    }
  else
    {
      prog->modv  = NULL;
    }

  prog->nfuncs = atol(strtok(NULL, " "));
  if ( prog->nfuncs > 0 )
    {
      prog->funcv  = (struct cone_func**) calloc(prog->nfuncs, sizeof(struct cone_func*));
      if ( prog->funcv == NULL )
	cone_error();
    }
  else
    {
      prog->funcv = NULL;
    }
  
  prog->ncsites = atol(strtok(NULL, " "));
  if ( prog->ncsites > 0 )
    {
      prog->csitev  = (struct cone_csite**) calloc(prog->ncsites, sizeof(struct cone_csite*));
      if ( prog->csitev == NULL )
	cone_error();
    }
  else
    {
      prog->csitev = NULL;
    }
  
  prog->ncnodes = atol(strtok(NULL, " "));
  if ( prog->ncnodes > 0 )
    {
      prog->cnodev  = (struct cone_cnode**) calloc(prog->ncnodes, sizeof(struct cone_cnode*));
      if ( prog->cnodev == NULL )
	cone_error();
    }
  else
    {
      prog->cnodev = NULL;
    }
 
  name = strtok(NULL, " ");
  prog->name = (char*) malloc(strlen(name) + 1);
  if ( prog->name == NULL )
    cone_error();
  memcpy(prog->name, name, strlen(name) + 1);
  
  for ( i = 0; i < prog->nmods; i++)
    {
      struct cone_mod* mod;

      if ( fgets(line, MAX_LINE_LENGTH, file) == NULL )
	cone_error();
        mod = cone_mod_create(line);
        prog->modv[cone_mod_id(mod)] = mod;
    }

  for ( i = 0; i < prog->nfuncs; i++)
    {
      struct cone_func* func;

      if ( fgets(line, MAX_LINE_LENGTH, file) == NULL )
	cone_error();
      func = cone_func_create(line);
      prog->funcv[cone_func_id(func)] = func;
    }

  for ( i = 0; i < prog->ncsites; i++)
    {
      struct cone_csite* csite;

      if ( fgets(line, MAX_LINE_LENGTH, file) == NULL )
	cone_error();
      csite = cone_csite_create(line);
      prog->csitev[cone_csite_id(csite)] = csite;
    }
    
 
  for ( i = 0; i < prog->ncnodes; i++)
    {
      struct cone_cnode* cnode;

      if ( fgets(line, MAX_LINE_LENGTH, file) == NULL )
	cone_error();
      cnode = cone_cnode_create(line);
      prog->cnodev[cone_cnode_id(cnode)] = cnode;
    }


  close(file);

  return prog;
}

void
cone_prog_delete(struct cone_prog* prog)
{
  int i;

  free(prog->name);

  for ( i = 0; i < prog->nmods; i++)
    cone_mod_delete(prog->modv[i]);
  for ( i = 0; i < prog->nfuncs; i++)
    cone_func_delete(prog->funcv[i]);
  for ( i = 0; i < prog->ncsites; i++)
    cone_csite_delete(prog->csitev[i]);
  for ( i = 0; i < prog->ncnodes; i++)
    cone_cnode_delete(prog->cnodev[i]);
 
  free(prog->modv);
  free(prog->funcv);
  free(prog->csitev);
  free(prog->cnodev);
 
  free(prog);
}

const char*      
cone_prog_name(struct cone_prog* prog)
{
  return prog->name;
}

long             
cone_prog_num_mods(struct cone_prog* prog)
{
  return prog->nmods;
}

long             
cone_prog_num_funcs(struct cone_prog* prog)
{
  return prog->nfuncs;
}

long             
cone_prog_num_csites(struct cone_prog* prog)
{
  return prog->ncsites;
}

long             
cone_prog_num_cnodes(struct cone_prog* prog)
{
  return prog->ncnodes;
}

struct cone_mod*   
cone_prog_mod(struct cone_prog* prog, long i)
{
  return prog->modv[i];
}

struct cone_func*  
cone_prog_func(struct cone_prog* prog, long i)
{
  return prog->funcv[i];
}

struct cone_csite* 
cone_prog_csite(struct cone_prog* prog, long i)
{
  return prog->csitev[i];
}

struct cone_cnode* 
cone_prog_cnode(struct cone_prog* prog, long i)
{
  return prog->cnodev[i];
}



void             
cone_prog_fprintf(FILE* stream, struct cone_prog* prog)
{
  int i;
  
  fprintf(stream, "PROGRAM %s %d %d %d %d\n", 
	  prog->name, 
	  prog->nmods,  
	  prog->nfuncs,  
	  prog->ncsites,
          prog->ncnodes);
  
  for ( i = 0; i < prog->nmods; i++)
    cone_mod_fprintf(stream, prog->modv[i]);
  for ( i = 0; i < prog->nfuncs; i++)
    cone_func_fprintf(stream, prog->funcv[i]);
  for ( i = 0; i < prog->ncsites; i++)
    cone_csite_fprintf(stream, prog->csitev[i]);
  for ( i = 0; i < prog->ncnodes; i++)
    cone_cnode_fprintf(stream, prog->cnodev[i]);
}








